package gestione_tessile_con_liste;

import java.util.*;
import java.text.*;

public class GestioneTessile {

	public static final int MAX_MATERIALI = 100;
	
	private Materiale[] materiali;
	private int numMateriali; // numero di materiali presenti effettivamente nel vettore
	private List<Fornitore> fornitori;
	
	public GestioneTessile(){
		this(new Materiale[MAX_MATERIALI],0,new ArrayList<Fornitore>());
	}
	
	public GestioneTessile(Materiale[] materiali, int n, List<Fornitore> fornitori){
		this.materiali = materiali;
		this.numMateriali = n;
		this.fornitori = fornitori;
	}
	
	// metodo 1)
	public Materiale cercaMateriale(int codiceM) {
		Materiale m = null;
		boolean trovato = false;
		for(int i=0; i<numMateriali && !trovato; i++)
			if(materiali[i].getCodice() == codiceM){
				trovato = true;
				m = materiali[i];
			}
		return m;	
	}
	
	// metodo 2)
	public DatiMateriali contaMaterialiPerSettori(){
		DatiMateriali dati;
		int numAbbigliamento=0, numArredamento=0;
		for(int i=0; i<numMateriali; i++)	
			if(materiali[i] instanceof MaterialeAbbigliamento)
				numAbbigliamento++;
			else
				numArredamento++;
		dati = new DatiMateriali(numAbbigliamento,numArredamento); // per restituire due valori,  necessario creare
																   // un oggetto di una classe che rappresenti una
																   // coppia di valori
		return dati;
	}
	
	
	/*
	// metodo 2) versione alternativa con creazione e aggiornamento di un oggetto di tipo DatiMateriali
	public DatiMateriali contaMaterialiPerSettori(){
		DatiMateriali dati = new DatiMateriali();
		for(int i=0; i<numMateriali; i++)	
			if(m instanceof MaterialeAbbigliamento)
				dati.incrementaAbbigliamento();
			else
				dati.incrementaArredamento();
		return dati;
	}
	*/
	
	// metodo 3)
	public void aggiornaFornitorePerMateriale(Fornitore fornitore, int codMateriale){
		fornitori.add(fornitore);
		// richiamo del metodo 1) per cercare il materiale corrispondente al codice dato
		Materiale m = cercaMateriale(codMateriale);
		if(m != null)
				m.setCodFornitore(fornitore.getCodice());
	}
	
	// metodo 4)
	public boolean aggiungiMateriale(Materiale materiale){
		boolean inserito = false;
		// verifico ci sia spazio nel vettore
		if(numMateriali < MAX_MATERIALI) { // oppure if(numMateriali < materiali.length)
			// verifico che il materiale non sia gi presente nel vettore
			Materiale m = cercaMateriale(materiale.getCodice());
			if(m == null) { // il materiale non  presente
				materiali[numMateriali++] = materiale;
				inserito = true;
			}
		}
		return inserito;
	}
	
		
	// metodo 5) con uso di metodi per l'accesso posizionale alle liste
	public int fornitoriPerCitta(String citta){
		int numStampati = 0;
		for(int i=0; i<fornitori.size();i++){
			Fornitore fornitore = fornitori.get(i);
			if(fornitore.getCitta().equals(citta)){
				System.out.println("Denominazione: " + fornitore.getDenominazione());
				System.out.println("Email: "+ fornitore.getEmail());
				System.out.println();
				numStampati++;
			}	
		}
		return numStampati;
	}
	
	/*
	// metodo 5 alternativo con uso di iteratori
	public int fornitoriPerCitta(String citta) {
		int numStampati = 0;
		Iterator<Fornitore> i = fornitori.iterator();
		while(i.hasNext()) {
			Fornitore f = i.next();
			if(f.getCitta().equalsIgnoreCase(citta)) {
				System.out.println("Denominazione: " + f.getDenominazione());
				System.out.println("Email: "+ f.getEmail());
				System.out.println();
				numStampati++;
			}
		}
		return numStampati;
	}
	*/

	// metodo 6)
	public int codiceFornitorePiuEconomico(String descrizione){
		int codiceFornitore = -1;
		double costoMinimo = Double.MAX_VALUE; // valore molto alto per consentire al primo costo di essere riconosciuto come il minimo corrente
		Materiale economico = null;
		
		for(int i=0; i<numMateriali; i++)
			if(materiali[i].getDescrizione().equals(descrizione))
				if (materiali[i].getCostoAlMetro() < costoMinimo){
					economico = materiali[i];
					costoMinimo = economico.getCostoAlMetro();
				}
		if(economico == null)
			System.out.println("Errore. Materiale con descrizione " + descrizione + " non trovato");
		else
			codiceFornitore = economico.getCodFornitore();
		return codiceFornitore;
	}
	
	
	// metodo 7) richiamo a metodo gi definito
	public Fornitore fornitorePiuEconomico(String descrizione){
		Fornitore f = null;
		int codFornitore = codiceFornitorePiuEconomico(descrizione);
		if(codFornitore != -1) {
			boolean trovato = false;
			for(int i=0; i<fornitori.size() && !trovato; i++)
				if(fornitori.get(i).getCodice() == codFornitore){
					trovato = true;
					f = fornitori.get(i); 
				}
		}
		return f;
	}
	
	/*
	// metodo 7) alternativo con richiamo al metodo di utilit cercaFornitore
		public Fornitore fornitorePiuEconomico(String descrizione){
			Fornitore f = null;
			int codFornitore = codiceFornitorePiuEconomico(descrizione);
			if(codFornitore != -1) 
				f = cercaFornitore(codFornitore);
			return f;
		}
	*/
	
	/*
	// metodo 7) alternativo con uso di iteratori
		public Fornitore fornitorePiuEconomico(String descrizione){
			Fornitore f = null;
			int codFornitore = codiceFornitorePiuEconomico(descrizione);
			if(codFornitore != -1){
				boolean trovato = false;
				Iterator<Fornitore> i = fornitori.iterator();
				while(i.hasNext() && !trovato){
					Fornitore fornitore = i.next();
					if(fornitore.getCodice() == codFornitore){
						trovato = true;
						f = fornitore; 
					}
				}
			}
			return f;
		}
		*/
	
	// metodo 8) con attenzione all'uso di cast necessari
	public void statisticheMateriali(){
		int totAbbigliamento=0;
		int totArredamento=0;
		int totTendaggi=0;
		int totDivani=0;
		int totBiancheria=0;
		
		for(int i=0; i<numMateriali; i++)
			if(materiali[i] instanceof MaterialeAbbigliamento)
				totAbbigliamento++;
			else{
				totArredamento++;
				// notare il cast obbligatorio perch getTipoArredamento  definito solo per MaterialeArredamento
				if(((MaterialeArredamento)materiali[i]).getTipoArredamento().equals("TENDAGGI"))
					totTendaggi++;
				else if(((MaterialeArredamento)materiali[i]).getTipoArredamento().equals("DIVANI"))
					totDivani++;
				else
					totBiancheria++;
			}
		DecimalFormat df = new DecimalFormat("0.0");
		System.out.print("La percentuale di materiali per l'abbigliamento : ");
		System.out.println(df.format((double)totAbbigliamento/numMateriali*100)+"%");
		System.out.print("La percentuale di materiali per l'arredamento : ");
		System.out.println(df.format((double)totArredamento/numMateriali*100)+"%");
		System.out.print("La percentuale di materiali di arredamento per i tendaggi : ");
		System.out.println(df.format((double)totTendaggi/totArredamento*100)+"%");
		System.out.print("La percentuale di materiali di arredamento per i divani : ");
		System.out.println(df.format((double)totDivani/totArredamento*100)+"%");
		System.out.print("La percentuale di materiali di arredamento per la biancheria : ");
		System.out.println(df.format((double)totBiancheria/totArredamento*100)+"%");
	}
	
	
	// metodo 9) uso del metodo polimorfo costoFornitura
	public double costoFornituraPerMateriale(int codiceMateriale, int metri){
		double costo=-1;
		Materiale m = cercaMateriale(codiceMateriale);
		if(m != null)
				costo = m.costoFornitura(metri);
		return costo;
	}
	
	// metodo 10) con uso di un metodo di utilit per la ricerca del fornitore nella lista dei fornitori
	public void fornitoriPerMateriali(){
		for(int i=0; i<numMateriali; i++) {
			System.out.println("Materiale " + (i+1) + ": " + materiali[i]);
			Fornitore fornitore = cercaFornitore(materiali[i].getCodFornitore());
			System.out.println("Fornitore: " + fornitore);
		}
	}
	
	// metodo di utilit per la ricerca di un fornitore nella lista dei fornitori in base al codice del fornitore
	private Fornitore cercaFornitore(int codiceFornitore) {
		Fornitore fornitore = null;
		boolean trovato = false;
		Iterator<Fornitore> i = fornitori.iterator();
		while(i.hasNext() && !trovato) {
			Fornitore f = i.next();
			if(f.getCodice() == codiceFornitore) {
				trovato = true;
				fornitore = f;
			}
		}
		return fornitore;
	}
}
